﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using GestioHospitalaria;
using GestioHospitalaria.Controllers;
using GestioHospitalaria.Tests;
using GestioHospitalaria.Tests.Models;
using GestioHospitalaria.Models;
using System.Web.Routing;
using System.Web;
using System.Security.Principal;
using GestioHospitalaria.Auxiliar;
using MvcContrib.TestHelper;

namespace GestioHospitalaria.Tests.Controllers
{
    [TestClass]
    public class FrequenciaControllerTest
    {
        //PREGUNTEM SOBRE EL MISSATGE DE LA PÀGINA INDEX
        [TestMethod]
        public void Index_Get_AsksForIndexViewMessage()
        {
            // Arrange
            var controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());

            // Act
            ViewResult result = controller.IndexListTest() as ViewResult;

            // Assert
            Assert.AreEqual("Index List Test", result.ViewBag.Message);
        }

        //VERIFIQUEM QUE REBEM TOTS ELS ELEMENTS DE LA TAULA DEL REPOSITORI
        [TestMethod]
        public void Index_Get_RetrievesAllFrequenciaFromRepository()
        {
            // Arrange
            Frequencia amdf1 = Auxiliar.Auxiliar.GetFrequenciaF(1);
            InMemoryGestioHospitalariaRepository repository = new InMemoryGestioHospitalariaRepository();
            repository.AddFrequencia(amdf1);
            var controller = GetFrequenciaController(repository);

            // Act
            var result = controller.IndexListTest();

            // Assert
            var model = (IEnumerable<Frequencia>)result.ViewData.Model;
            CollectionAssert.Contains(model.ToList(), amdf1);
        }

        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxCREATExxxxxxxxxxxxxxxxxxxxxxxxx


        //AGREGEM UN ELEMENT A LA TAULA DEL REPOSITORI
        [TestMethod]
        public void Create_Post_PutsValidElementIntoRepository()
        {
            // Arrange
            InMemoryGestioHospitalariaRepository repository = new InMemoryGestioHospitalariaRepository();
            FrequenciaController controller = GetFrequenciaController(repository);
            Frequencia Frequencia = Auxiliar.Auxiliar.GetFrequenciaID_1();

            // Act
            controller.CreateTest(Frequencia);

            // Assert
            IEnumerable<Frequencia> Frequencias = repository.GetAllFrequencia();
            Assert.IsTrue(Frequencias.Contains(Frequencia));
        }

        //VERIFIQUEM QUE ENS PORTA A LA PAGINA CORRECTA EN CAS DE CREAR UN ELEMENT CORRECTAMENT
        [TestMethod]
        public void Create_Post_ReturnsRedirectOnSuccess()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();

            // Act
            var result = (RedirectToRouteResult)controller.CreateTest(model);

            // Assert
            Assert.AreEqual("IndexListTest", result.RouteValues["action"]);
        }

        //EN CAS DE QUE ELS ELEMENTS A CREAR ESTIGUIN MALAMENT VERIFIQUEM A ON ENS PORTA
        [TestMethod]
        public void Create_Post_ReturnsViewIfModelStateIsNotValid()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            controller.ModelState.AddModelError("", "mock error message");
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaF(1);

            // Act
            var result = (ViewResult)controller.CreateTest(model);

            // Assert
            Assert.AreEqual("Error en la creació", result.ViewBag.Message);
        }

        //EN CAS DE QUE EL REPOSITORI LLANCI UNA EXCEPCIÓ VERIFIQUEM A ON ENS PORTA
        [TestMethod]
        public void Create_Post_ReturnsViewIfRepositoryThrowsException()
        {
            // Arrange
            InMemoryGestioHospitalariaRepository repository = new InMemoryGestioHospitalariaRepository();
            Exception exception = new Exception();
            repository.ExceptionToThrow = exception;
            FrequenciaController controller = GetFrequenciaController(repository);
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();

            // Act
            var result = (ViewResult)controller.CreateTest(model);

            // Assert
            Assert.AreEqual("Error en la creació", result.ViewBag.Message);
            ModelState modelState = result.ViewData.ModelState[""];
            Assert.IsNotNull(modelState);
            Assert.IsTrue(modelState.Errors.Any());
            Assert.AreEqual(exception, modelState.Errors[0].Exception);
        }
        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxCREATExxxxxxxxxxxxxxxxxxxxxxxxx

        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxEDITxxxxxxxxxxxxxxxxxxxxxxxxx

        //VERIFIQUEM QUE ENS PORTA A LA PAGINA CORRECTA EN CAS DE VOLER EDITAR UN ELEMENT
        [TestMethod]
        public void Edit_Get_ReturnsRedirectOnSuccess()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());

            // Act
            var result = (ViewResult)controller.EditTest(1);

            // Assert
            Assert.AreEqual("Entrada Edició", result.ViewBag.Message);
        }

        //VERIFIQUEM QUE ENS PORTA A LA PAGINA CORRECTA EN CAS DE EDITAR UN ELEMENT CORRECTAMENT
        [TestMethod]
        public void Edit_Post_ReturnsRedirectOnSuccess()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();
            // Act
            var result = (RedirectToRouteResult)controller.EditTest(model);

            // Assert
            Assert.AreEqual("IndexListTest", result.RouteValues["action"]);
        }

        //EN CAS DE QUE ELS ELEMENTS A EDITAR ESTIGUIN MALAMENT VERIFIQUEM A ON ENS PORTA
        [TestMethod]
        public void Edit_Post_ReturnsViewIfModelStateIsNotValid()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            controller.ModelState.AddModelError("", "mock error message");
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaF(1);

            // Act
            var result = (ViewResult)controller.EditTest(model);

            // Assert
            Assert.AreEqual("Error en la Edició", result.ViewBag.Message);
        }

        //EN CAS DE QUE EL REPOSITORI LLANCI UNA EXCEPCIÓ VERIFIQUEM A ON ENS PORTA
        [TestMethod]
        public void Edit_Post_ReturnsViewIfRepositoryThrowsException()
        {
            // Arrange
            InMemoryGestioHospitalariaRepository repository = new InMemoryGestioHospitalariaRepository();
            Exception exception = new Exception();
            repository.ExceptionToThrow = exception;
            FrequenciaController controller = GetFrequenciaController(repository);
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();

            // Act
            var result = (ViewResult)controller.EditTest(model);

            // Assert
            Assert.AreEqual("Error en la Edició", result.ViewBag.Message);
            ModelState modelState = result.ViewData.ModelState[""];
            Assert.IsNotNull(modelState);
            Assert.IsTrue(modelState.Errors.Any());
            Assert.AreEqual(exception, modelState.Errors[0].Exception);
        }

        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxEDITxxxxxxxxxxxxxxxxxxxxxxxxx

        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxDELETExxxxxxxxxxxxxxxxxxxxxxxxx

        //VERIFIQUEM QUE ENS PORTA A LA PAGINA CORRECTA EN CAS DE VOLER ELIMINAR UN ELEMENT
        [TestMethod]
        public void Delete_Get_ReturnsRedirectOnSuccess()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());

            // Act
            var result = (ViewResult)controller.DeleteTest(1);

            // Assert
            Assert.AreEqual("Eliminació", result.ViewBag.Message);
        }

        //VERIFIQUEM QUE ENS PORTA A LA PAGINA CORRECTA EN CAS D'ELIMINAR UN ELEMENT CORRECTAMENT
        [TestMethod]
        public void Delete_Post_ReturnsRedirectOnSuccess()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();
            // Act
            var result = (RedirectToRouteResult)controller.DeleteTest(model);

            // Assert
            Assert.AreEqual("IndexListTest", result.RouteValues["action"]);
        }

        //EN CAS DE QUE ELS ELEMENTS A ELIMINAR ESTIGUIN MALAMENT VERIFIQUEM A ON ENS PORTA
        [TestMethod]
        public void Delete_Post_ReturnsViewIfModelStateIsNotValid()
        {
            // Arrange
            FrequenciaController controller = GetFrequenciaController(new InMemoryGestioHospitalariaRepository());
            controller.ModelState.AddModelError("", "mock error message");
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaF(1);

            // Act
            var result = (ViewResult)controller.DeleteTest(model);

            // Assert
            Assert.AreEqual("Error en l'eliminació", result.ViewBag.Message);
        }
        //xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxDELETExxxxxxxxxxxxxxxxxxxxxxxxx
        [TestMethod]//verifiquem que qui no te permís d'entrada surt donat que el tipus d'usuari no es correcte
        public void Ask_For_UserSessionNoOK()
        {
            // Arrange
            FrequenciaController controller = new FrequenciaController(new InMemoryGestioHospitalariaRepository());
            TestControllerBuilder builder = new TestControllerBuilder();
            builder.InitializeController(controller);
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();
            // Act
            controller.HttpContext.Session["IDTipusEmpleat"] = 5;
            var resultIndex = (RedirectToRouteResult)controller.Index(null, null, null, 0);
            var resultDetails = (RedirectToRouteResult)controller.Details();
            var resultCreate = (RedirectToRouteResult)controller.Create();
            var resultCreate1 = (RedirectToRouteResult)controller.Create(model);
            var resultEdit = (RedirectToRouteResult)controller.Edit();
            var resultEdit1 = (RedirectToRouteResult)controller.Edit(model);
            var resultDelete = (RedirectToRouteResult)controller.Delete();
            var resultDelete1 = (RedirectToRouteResult)controller.Delete(model.ID_Frequencia);
            // Assert
            Assert.AreEqual("Logout", resultIndex.RouteValues["action"]);
            Assert.AreEqual("Logout", resultDetails.RouteValues["action"]);
            Assert.AreEqual("Logout", resultCreate.RouteValues["action"]);
            Assert.AreEqual("Logout", resultCreate1.RouteValues["action"]);
            Assert.AreEqual("Logout", resultEdit.RouteValues["action"]);
            Assert.AreEqual("Logout", resultEdit1.RouteValues["action"]);
            Assert.AreEqual("Logout", resultDelete.RouteValues["action"]);
            Assert.AreEqual("Logout", resultDelete1.RouteValues["action"]);
        }
        //verifiquem que qui te permís d'entrada, però genera un error a causa de que es treballa amb SQLServer i no pot convertir 
        //el tipus InMemoryGestioHospitalariaRepository en EntityGestioHospitalariaManagerRepository
        [TestMethod]
        public void Ask_For_UserSessionOKAndErrorEntityGestioHospitalariaManagerRepository()
        {
            // Arrange
            FrequenciaController controller = new FrequenciaController(new InMemoryGestioHospitalariaRepository());
            TestControllerBuilder builder = new TestControllerBuilder();
            builder.InitializeController(controller);
            Frequencia model = Auxiliar.Auxiliar.GetFrequenciaID_1();

            controller.HttpContext.Session["IDTipusEmpleat"] = 3;
            var resultIndex = (RedirectToRouteResult)controller.Index(null, null, null, 0);
            Assert.AreEqual("Logout", resultIndex.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
                controller.HttpContext.Session["ErrorMes"]);

            var resultDetails = (RedirectToRouteResult)controller.Details();
            Assert.AreEqual("Logout", resultDetails.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultCreate = (RedirectToRouteResult)controller.Create();
            Assert.AreEqual("Logout", resultCreate.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultCreate1 = (RedirectToRouteResult)controller.Create(model);
            Assert.AreEqual("Logout", resultCreate1.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultEdit = (RedirectToRouteResult)controller.Edit();
            Assert.AreEqual("Logout", resultEdit.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultEdit1 = (RedirectToRouteResult)controller.Edit(model);
            Assert.AreEqual("Logout", resultEdit1.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultDelete = (RedirectToRouteResult)controller.Delete();
            Assert.AreEqual("Logout", resultDelete.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);

            var resultDelete1 = (RedirectToRouteResult)controller.Delete(model.ID_Frequencia);
            Assert.AreEqual("Logout", resultDelete1.RouteValues["action"]);
            Assert.AreEqual("No se puede convertir un objeto de tipo 'GestioHospitalaria.Tests.Models.InMemoryGestioHospitalariaRepository' al tipo 'GestioHospitalaria.Models.EntityGestioHospitalariaManagerRepository'.",
            controller.HttpContext.Session["ErrorMes"]);
        }
        //ENS SERVEIX PER A CREAR EL CONTROLADOR
        private static FrequenciaController GetFrequenciaController(IGestioHospitalariaRepository repository)
        {
            FrequenciaController controller = new FrequenciaController(repository);

            controller.ControllerContext = new ControllerContext()
            {
                Controller = controller,
                RequestContext = new RequestContext(new MockHttpContext(), new RouteData())
            };
            return controller;
        }
        private class MockHttpContext : HttpContextBase
        {
            private readonly IPrincipal _user = new GenericPrincipal(
                     new GenericIdentity("someUser"), null /* roles */);

            public override IPrincipal User
            {
                get
                {
                    return _user;
                }
                set
                {
                    base.User = value;
                }
            }
        }

    }
}
