﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;
using System.Data.Objects.SqlClient;

namespace GestioHospitalaria.Controllers
{
    public class AdministrarFarmacController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public AdministrarFarmacController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllAdministrarFarmac());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(AdministrarFarmac administrarfarmac)
        {
            try {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewAdministrarFarmac(administrarfarmac);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

                ViewBag.Message = "Error en la creació";
                return View(administrarfarmac);
        }

        public ActionResult EditTest(int id)
        {
            var administrarfarmac = _repository.GetAdministrarFarmacByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(administrarfarmac);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(AdministrarFarmac administrarFarmac)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteAdministrarFarmac(administrarFarmac.ID_AdministrarFarmac);
                    _repository.CreateNewAdministrarFarmac(administrarFarmac);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició"; 
            return View(administrarFarmac);
         }

        public ActionResult DeleteTest(int id)
        {
            var administrarFarmac = _repository.GetAdministrarFarmacByID(id);
            ViewBag.Message = "Eliminació";
            return View(administrarFarmac);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(AdministrarFarmac administrarFarmac)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteAdministrarFarmac(administrarFarmac.ID_AdministrarFarmac);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(administrarFarmac);
        }


    //fi test

        //
        // GET: /AdministrarFarmac/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var administrarfarmac = db.AdministrarFarmac.Include(a => a.Dosi).Include(a => a.Habitacio).Include(a => a.Medicament).Include(a => a.Usuari);
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        administrarfarmac = administrarfarmac.Where(s => s.Habitacio.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_AdministrarFarmac":
                            administrarfarmac = administrarfarmac.OrderBy(s => s.ID_AdministrarFarmac);
                            break;
                        case "DataHora":
                            administrarfarmac = administrarfarmac.OrderBy(s => s.DataHora);
                            break;
                        case "ID_Habitacio":
                            administrarfarmac = administrarfarmac.OrderBy(s => s.Habitacio.ID_Habitacio);
                            break;
                        default:
                            administrarfarmac = administrarfarmac.OrderBy(s => s.ID_AdministrarFarmac);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(administrarfarmac.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch(Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }


        //
        // GET: /AdministrarFarmac/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    AdministrarFarmac administrarfarmac = db.AdministrarFarmac.Find(id);
                    if (administrarfarmac == null)
                    {
                        return HttpNotFound();
                    }
                    return View(administrarfarmac);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /AdministrarFarmac/Create

        public ActionResult Create()
        {
            try
            {

                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari

                    ViewBag.Datahora = DateTime.Now.ToString();
                    ViewBag.IDUsuari = u.ID_Usuari;
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();

                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat");
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio");
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio");
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", u.ID_Usuari);
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /AdministrarFarmac/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(AdministrarFarmac administrarfarmac)
        {
            try
            {
                Boolean error = false;
                string cadenaErrors = "";

                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    cadenaErrors = tractamentOK(administrarfarmac);
                    if (cadenaErrors != "") error = true;
                    try
                    {
                        if (ModelState.IsValid && !error)//si no tenim errors en la verificació
                        {
                            db.AdministrarFarmac.Add(administrarfarmac);
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                    }
                    catch (Exception ex)
                    {
                        ModelState.AddModelError("", ex);
                        ViewBag.Message = "Error en la creació";
                    }
                    ViewBag.Datahora = DateTime.Now.ToString();
                    ViewBag.IDUsuari = administrarfarmac.ID_Usuari;
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();

                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", administrarfarmac.ID_Dosi);
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", administrarfarmac.ID_Habitacio);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", administrarfarmac.ID_Medicament);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", administrarfarmac.ID_Usuari);
                    ModelState.AddModelError("", cadenaErrors);
                    return View(administrarfarmac);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /AdministrarFarmac/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    AdministrarFarmac administrarfarmac = db.AdministrarFarmac.Find(id);
                    if (administrarfarmac == null)
                    {
                        return HttpNotFound();
                    }
                    ViewBag.Datahora = DateTime.Now.ToString();
                    ViewBag.IDUsuari = administrarfarmac.ID_Usuari;
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();

                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", administrarfarmac.ID_Dosi);
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", administrarfarmac.ID_Habitacio);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", administrarfarmac.ID_Medicament);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", administrarfarmac.ID_Usuari);
                    return View(administrarfarmac);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /AdministrarFarmac/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(AdministrarFarmac administrarfarmac)
        {
            try
            {
                Boolean error = false;
                string cadenaErrors = "";

                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    cadenaErrors = tractamentOK(administrarfarmac);
                    if (cadenaErrors != "") error = true;

                    if (ModelState.IsValid && !error)//si no tenim errors en la verificació
                    {
                        db.Entry(administrarfarmac).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ViewBag.Datahora = DateTime.Now.ToString();
                    ViewBag.IDUsuari = administrarfarmac.ID_Usuari;
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();

                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", administrarfarmac.ID_Dosi);
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", administrarfarmac.ID_Habitacio);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", administrarfarmac.ID_Medicament);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", administrarfarmac.ID_Usuari);
                    ModelState.AddModelError("", cadenaErrors);
                    return View(administrarfarmac);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }
        //
        // GET: /AdministrarFarmac/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    AdministrarFarmac administrarfarmac = db.AdministrarFarmac.Find(id);
                    if (administrarfarmac == null)
                    {
                        return HttpNotFound();
                    }
                    return View(administrarfarmac);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /AdministrarFarmac/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    AdministrarFarmac administrarfarmac = db.AdministrarFarmac.Find(id);
                    db.AdministrarFarmac.Remove(administrarfarmac);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(administrarfarmac);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }


        //mètode que valida la correcte administració del tractament establert per el metge
        public string tractamentOK(AdministrarFarmac administrarfarmac)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                string sortida = "";
                var ingres = from i in db.Ingres where i.ID_Habitacio == administrarfarmac.ID_Habitacio && !i.DataFi.HasValue select i;
                if (ingres.Count() <= 0)
                {
                    sortida = "Actualment no hi ha cap pacient assignat a aquesta habitació";
                }
                else//verifiquem el tractament que sigui correcte
                {
                    Ordre ordre = db.Ordre.Find(ingres.First().ID_Ordre);//cerquem l'ordre d'hospitalització per a obtenir el pacient
                    var tractament = from t in db.Tractament where t.ID_Pacient == ordre.ID_Pacient && t.ID_Medicament == administrarfarmac.ID_Medicament && t.ID_Dosi == administrarfarmac.ID_Dosi select t;
                    if (tractament.Count() <= 0)
                    {
                        sortida = "No existeix cap tractament amb aquestes característiques assignada a aquest pacient";

                    }
                    else
                    {// +-10 minuts marge de l'administració del tractament
                        //montem 
                        Boolean okTractamentDateTime = false;
                        foreach (Tractament t in tractament)
                        {
                            DateTime minDate = t.DataInici;
                            DateTime maxDate = t.DataFi;
                            DateTime actDate = minDate;


                            int sumDies = t.Frequencia.Dies;
                            int sumHores = t.Frequencia.Hores;
                            int sumMesos = t.Frequencia.Mesos;
                            int sumMinuts = t.Frequencia.Minuts;
                            int sumSetmanes = t.Frequencia.Setmanes;


                            while (actDate <= maxDate)
                            {//donem 20 minuts de marge
                                DateTime actDatePos = actDate.AddMinutes(10);
                                DateTime actDateNeg = actDate.AddMinutes(-10);

                                if (administrarfarmac.DataHora >= actDateNeg && administrarfarmac.DataHora <= actDatePos) okTractamentDateTime = true;
                                //s'ha administrat el fàcmec bé
                                actDate = actDate.AddDays(7 * sumSetmanes + sumDies);//sumen a la data els dies de freqüència
                                actDate = actDate.AddHours(sumHores);//sumen a la data les hores de freqüència
                                actDate = actDate.AddMonths(sumMesos);//sumen a la data els mesos de freqüència
                                actDate = actDate.AddMinutes(sumMinuts);//sumen a la data els minuts de freqüència

                            }
                        }
                        if (!okTractamentDateTime) sortida = "S'ha administrat el fàrmac fora de la temporització establerta";

                    }
                }


                return sortida;
            }
            catch
            {
                return "Error no controlat";
            }

        }
    }
}