﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class DosiController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public DosiController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //
        // GET: /Dosi/
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllDosi());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Dosi Dosi)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewDosi(Dosi);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Dosi);
        }

        public ActionResult EditTest(int id)
        {
            var Dosi = _repository.GetDosiByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Dosi);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Dosi Dosi)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteDosi(Dosi.ID_Dosi);
                    _repository.CreateNewDosi(Dosi);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Dosi);
        }

        public ActionResult DeleteTest(int id)
        {
            var Dosi = _repository.GetDosiByID(id);
            ViewBag.Message = "Eliminació";
            return View(Dosi);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Dosi Dosi)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteDosi(Dosi.ID_Dosi);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Dosi);
        }


        //fi test
        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {

                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var dosi = from s in db.Dosi
                               select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        dosi = dosi.Where(s => s.Unitat.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Dosi":
                            dosi = dosi.OrderBy(s => s.ID_Dosi);
                            break;
                        case "Unitat":
                            dosi = dosi.OrderBy(s => s.Unitat);
                            break;
                        case "Valor":
                            dosi = dosi.OrderBy(s => s.Valor);
                            break;
                        default:
                            dosi = dosi.OrderBy(s => s.ID_Dosi);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(dosi.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Dosi/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Dosi dosi = db.Dosi.Find(id);
                    if (dosi == null)
                    {
                        return HttpNotFound();
                    }
                    return View(dosi);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Dosi/Create

        public ActionResult Create()
        {
            try
            {

                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Dosi/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Dosi dosi)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid)
                    {
                        db.Dosi.Add(dosi);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(dosi);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Dosi/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {

                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Dosi dosi = db.Dosi.Find(id);
                    if (dosi == null)
                    {
                        return HttpNotFound();
                    }
                    return View(dosi);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Dosi/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Dosi dosi)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid)
                    {
                        db.Entry(dosi).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    return View(dosi);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Dosi/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Dosi dosi = db.Dosi.Find(id);
                    if (dosi == null)
                    {
                        return HttpNotFound();
                    }
                    return View(dosi);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Dosi/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Dosi dosi = db.Dosi.Find(id);
                    db.Dosi.Remove(dosi);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(dosi);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}