﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;


namespace GestioHospitalaria.Controllers
{
    public class EstatReservaController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public EstatReservaController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllEstatReserva());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(EstatReserva EstatReserva)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewEstatReserva(EstatReserva);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(EstatReserva);
        }

        public ActionResult EditTest(int id)
        {
            var EstatReserva = _repository.GetEstatReservaByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(EstatReserva);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(EstatReserva EstatReserva)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteEstatReserva(EstatReserva.ID_EstatReserva);
                    _repository.CreateNewEstatReserva(EstatReserva);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(EstatReserva);
        }

        public ActionResult DeleteTest(int id)
        {
            var EstatReserva = _repository.GetEstatReservaByID(id);
            ViewBag.Message = "Eliminació";
            return View(EstatReserva);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(EstatReserva EstatReserva)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteEstatReserva(EstatReserva.ID_EstatReserva);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(EstatReserva);
        }


        //fi test

        //
        // GET: /EstatReserva/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();


                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var estatReserva = from s in db.EstatReserva
                                       select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        estatReserva = estatReserva.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_EstatReserva":
                            estatReserva = estatReserva.OrderBy(s => s.ID_EstatReserva);
                            break;
                        case "Descripcio":
                            estatReserva = estatReserva.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            estatReserva = estatReserva.OrderBy(s => s.ID_EstatReserva);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(estatReserva.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /EstatReserva/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    EstatReserva estatReserva = db.EstatReserva.Find(id);
                    if (estatReserva == null)
                    {
                        return HttpNotFound();
                    }
                    return View(estatReserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /EstatReserva/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /EstatReserva/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(EstatReserva estatReserva)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();


                    if (ModelState.IsValid)
                    {
                        db.EstatReserva.Add(estatReserva);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(estatReserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /EstatReserva/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    EstatReserva estatReserva = db.EstatReserva.Find(id);
                    if (estatReserva == null)
                    {
                        return HttpNotFound();
                    }
                    return View(estatReserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /EstatReserva/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(EstatReserva estatReserva)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid && estatReserva.ID_EstatReserva > 3)//evitem modificar dades reservades
                    {
                        db.Entry(estatReserva).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ModelState.AddModelError("", "No es pot modificar una dada reservada del sistema");
                    return View(estatReserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /EstatReserva/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    EstatReserva estatReserva = db.EstatReserva.Find(id);
                    if (estatReserva == null)
                    {
                        return HttpNotFound();
                    }
                    return View(estatReserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /EstatReserva/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    EstatReserva estatReserva = db.EstatReserva.Find(id);
                    if (estatReserva.ID_EstatReserva > 3)//evitem esborrar dades reservades
                    {
                        db.EstatReserva.Remove(estatReserva);
                        try
                        {
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        catch
                        {
                            ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                            return View(estatReserva);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", "No es pot esborrar una dada reservada del sistema");
                        return View(estatReserva);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}