﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class FrequenciaController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public FrequenciaController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllFrequencia());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Frequencia Frequencia)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewFrequencia(Frequencia);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Frequencia);
        }

        public ActionResult EditTest(int id)
        {
            var Frequencia = _repository.GetFrequenciaByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Frequencia);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Frequencia Frequencia)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteFrequencia(Frequencia.ID_Frequencia);
                    _repository.CreateNewFrequencia(Frequencia);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Frequencia);
        }

        public ActionResult DeleteTest(int id)
        {
            var Frequencia = _repository.GetFrequenciaByID(id);
            ViewBag.Message = "Eliminació";
            return View(Frequencia);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Frequencia Frequencia)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteFrequencia(Frequencia.ID_Frequencia);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Frequencia);
        }


        //fi test
        //
        // GET: /Frequencia/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();


                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var frequencia = from s in db.Frequencia
                                     select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        frequencia = frequencia.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Frequencia":
                            frequencia = frequencia.OrderBy(s => s.ID_Frequencia);
                            break;
                        case "Unitat":
                            frequencia = frequencia.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            frequencia = frequencia.OrderBy(s => s.ID_Frequencia);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(frequencia.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Frequencia/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Frequencia frequencia = db.Frequencia.Find(id);
                    if (frequencia == null)
                    {
                        return HttpNotFound();
                    }
                    return View(frequencia);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Frequencia/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Frequencia/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Frequencia frequencia)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Frequencia.Add(frequencia);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(frequencia);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Frequencia/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Frequencia frequencia = db.Frequencia.Find(id);
                    if (frequencia == null)
                    {
                        return HttpNotFound();
                    }
                    return View(frequencia);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Frequencia/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Frequencia frequencia)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Entry(frequencia).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    return View(frequencia);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Frequencia/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Frequencia frequencia = db.Frequencia.Find(id);
                    if (frequencia == null)
                    {
                        return HttpNotFound();
                    }
                    return View(frequencia);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Frequencia/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2") || Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Frequencia frequencia = db.Frequencia.Find(id);
                    db.Frequencia.Remove(frequencia);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(frequencia);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}