﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;

using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class HabitacioController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public HabitacioController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllHabitacio());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Habitacio Habitacio)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewHabitacio(Habitacio);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Habitacio);
        }

        public ActionResult EditTest(int id)
        {
            var Habitacio = _repository.GetHabitacioByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Habitacio);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Habitacio Habitacio)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteHabitacio(Habitacio.ID_Habitacio);
                    _repository.CreateNewHabitacio(Habitacio);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Habitacio);
        }

        public ActionResult DeleteTest(int id)
        {
            var Habitacio = _repository.GetHabitacioByID(id);
            ViewBag.Message = "Eliminació";
            return View(Habitacio);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Habitacio Habitacio)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteHabitacio(Habitacio.ID_Habitacio);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Habitacio);
        }


        //fi test

        //
        // GET: /Habitacio/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var habitacio = from s in db.Habitacio
                                    select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        habitacio = habitacio.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Habitacio":
                            habitacio = habitacio.OrderBy(s => s.ID_Habitacio);
                            break;
                        case "Descripcio":
                            habitacio = habitacio.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            habitacio = habitacio.OrderBy(s => s.ID_Habitacio);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(habitacio.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Habitacio/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Habitacio habitacio = db.Habitacio.Find(id);
                    if (habitacio == null)
                    {
                        return HttpNotFound();
                    }
                    return View(habitacio);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Habitacio/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Habitacio/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Habitacio habitacio)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Habitacio.Add(habitacio);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(habitacio);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Habitacio/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Habitacio habitacio = db.Habitacio.Find(id);
                    if (habitacio == null)
                    {
                        return HttpNotFound();
                    }
                    return View(habitacio);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Habitacio/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Habitacio habitacio)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Entry(habitacio).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    return View(habitacio);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Habitacio/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Habitacio habitacio = db.Habitacio.Find(id);
                    if (habitacio == null)
                    {
                        return HttpNotFound();
                    }
                    return View(habitacio);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Habitacio/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Habitacio habitacio = db.Habitacio.Find(id);
                    db.Habitacio.Remove(habitacio);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(habitacio);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}