﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class IngresController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public IngresController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllIngres());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Ingres Ingres)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewIngres(Ingres);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Ingres);
        }

        public ActionResult EditTest(int id)
        {
            var Ingres = _repository.GetIngresByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Ingres);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Ingres Ingres)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteIngres(Ingres.ID_Ingres);
                    _repository.CreateNewIngres(Ingres);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Ingres);
        }

        public ActionResult DeleteTest(int id)
        {
            var Ingres = _repository.GetIngresByID(id);
            ViewBag.Message = "Eliminació";
            return View(Ingres);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Ingres Ingres)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteIngres(Ingres.ID_Ingres);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Ingres);
        }


        //fi test
        //
        // GET: /Ingres/


        public ActionResult IndexOpen(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var ingres = db.Ingres.Include(i => i.Habitacio).Include(i => i.Ordre).Include(i => i.Usuari);
                    ingres = ingres.Where(s => !s.DataFi.HasValue);//busquem les obertes
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        if (searchString == null) searchString = currentFilter;
                    }



                    if (!String.IsNullOrEmpty(searchString))
                    {
                        ingres = ingres.Where(s => s.Ordre.Pacient.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Pacient.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Pacient.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }

                    switch (sortOrder)
                    {
                        case "ID_Ingres":
                            ingres = ingres.OrderBy(s => s.ID_Ingres);
                            break;
                        case "ID_Pacient":
                            ingres = ingres.OrderBy(s => s.Ordre.ID_Pacient);
                            break;
                        case "ID_Habitacio":
                            ingres = ingres.OrderBy(s => s.ID_Habitacio);
                            break;
                        default:
                            ingres = ingres.OrderBy(s => s.ID_Ingres);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(ingres.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        public ActionResult Index(string sortOrder, string currentFilter, string searchString,  int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var ingres = db.Ingres.Include(i => i.Habitacio).Include(i => i.Ordre).Include(i => i.Usuari);

                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        if (searchString == null) searchString = currentFilter;
                    }



                    if (!String.IsNullOrEmpty(searchString))
                    {
                        ingres = ingres.Where(s => s.Ordre.Pacient.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Pacient.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Pacient.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }

                    switch (sortOrder)
                    {
                        case "ID_Ingres":
                            ingres = ingres.OrderBy(s => s.ID_Ingres);
                            break;
                        case "ID_Pacient":
                            ingres = ingres.OrderBy(s => s.Ordre.ID_Pacient);
                            break;
                        case "ID_Habitacio":
                            ingres = ingres.OrderBy(s => s.ID_Habitacio);
                            break;
                        default:
                            ingres = ingres.OrderBy(s => s.ID_Ingres);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(ingres.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ingres/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Ingres ingres = db.Ingres.Find(id);
                    if (ingres == null)
                    {
                        return HttpNotFound();
                    }
                    return View(ingres);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ingres/Create

        public ActionResult Create(string id = "")
        {
            try
            {

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    if (id != "")//mirem si hem seleccionat una ordre d'ingrés
                    {
                        db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                        var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari

                        Ordre ordre = db.Ordre.Find(Convert.ToInt32(id));//busquem les dades de l'ordre
                        ViewBag.IDUsuari = u.ID_Usuari;
                        ViewBag.IDOrdre = id;
                        //és un protopip i no verifiquem que la data d'inici de l'ordre sigui la data del dia d'avuí, seria:
                        //if(ordre.DataHoraInici.Date == DateTime.Today.Date)
                        //en cas de no cumplir enviem un missatge que indiqui que la data de l'ingres sol·licitat no es l d'avuí llavors no     
                        //es pot realitzar l'ingrés.
                        ViewBag.DataInici = DateTime.Now.ToString();
                        //Cerquem una habitació lliure son aquelles que no tenen un ingrés associat amb una data fi en blanc
                        var habitacio = db.Habitacio.Where(h => !db.Ingres.Any(i => h.ID_Habitacio == i.ID_Habitacio && !i.DataFi.HasValue));
                        if (habitacio.Count() > 0) //mirem si queden habitacions lliures
                        {
                            ViewBag.IDHabitacio = habitacio.First().ID_Habitacio;//donem la primera habitació
                            ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", habitacio.First().ID_Habitacio);
                            ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", id);
                            ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", u.ID_Usuari);

                            return View();
                        }
                        else
                        {
                            Session["Error"] = "No queden habitacions lliures";

                            return RedirectToAction("Index", "Pacient");

                        }
                    }
                    else//començem el procés de selecció d'ordre
                    {
                        return RedirectToAction("Index", "Pacient");

                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }

            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ingres/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Ingres ingres)
        {//COM QUE NO HI HA CAP VARIACIÓ DE DADES POSSIBLES
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    ViewBag.IDUsuari = ingres.ID_Usuari;
                    ViewBag.IDOrdre = ingres.ID_Ordre;
                    ViewBag.DataInici = ingres.DataInici.ToString();
                    ViewBag.IDHabitacio = ingres.ID_Habitacio;
                    if (ModelState.IsValid && !isAdmitted(ingres))
                    {
                        db.Ingres.Add(ingres);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", ingres.ID_Habitacio);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", ingres.ID_Ordre);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", ingres.ID_Usuari);
                    ModelState.AddModelError("", "El pacient ja es troba ingressat");
                    return View(ingres);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ingres/Edit/5

        public ActionResult Edit(int id = 0)
        {//L'EDIT SERVIRÀ PER A TANCAR L'INGRÉS, ÉS A DIR, EL PACIENT MARXA
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    Ingres ingres = db.Ingres.Find(id);
                    ViewBag.IDUsuari = ingres.ID_Usuari;
                    ViewBag.IDOrdre = ingres.ID_Ordre;
                    ViewBag.DataInici = ingres.DataInici.ToString();
                    ViewBag.IDHabitacio = ingres.ID_Habitacio;
                    if (ingres == null)
                    {
                        return HttpNotFound();
                    }
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", ingres.ID_Habitacio);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", ingres.ID_Ordre);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", ingres.ID_Usuari);
                    ViewBag.DATA_FI = DateTime.Now;
                    return View(ingres);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ingres/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Ingres ingres)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    ViewBag.IDUsuari = ingres.ID_Usuari;
                    ViewBag.IDOrdre = ingres.ID_Ordre;
                    ViewBag.DataInici = ingres.DataInici.ToString();
                    ViewBag.IDHabitacio = ingres.ID_Habitacio;
                    ViewBag.DATA_FI = DateTime.Now;
                    if (ModelState.IsValid)
                    {
                        db.Entry(ingres).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ViewBag.ID_Habitacio = new SelectList(db.Habitacio, "ID_Habitacio", "Descripcio", ingres.ID_Habitacio);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", ingres.ID_Ordre);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", ingres.ID_Usuari);
                    return View(ingres);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ingres/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    Ingres ingres = db.Ingres.Find(id);
                    if (ingres == null)
                    {
                        return HttpNotFound();
                    }
                    return View(ingres);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ingres/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    Ingres ingres = db.Ingres.Find(id);
                    db.Ingres.Remove(ingres);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(ingres);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }
        //mirem si el pacient ja està ingresat
        public Boolean isAdmitted(Ingres ingres_)
        {
            Boolean isadmitted = false;
            try
            {

                var ingres = db.Ingres.Include(i => i.Habitacio).Include(i => i.Ordre).Include(i => i.Usuari);
                var ordre = db.Ordre.Find(ingres_.ID_Ordre);
                ingres = ingres.Where(s => !s.DataFi.HasValue);//busquem les obertes
                ingres = ingres.Where(i => i.Ordre.ID_Pacient == ordre.ID_Pacient);//busquem un ingrés obert del pacient
                if (ingres.Count() <= 0) //si no hi ha un registre
                    isadmitted = false;
                else
                    isadmitted = true;
            }
            catch (Exception e)
            {
                return true;//en cas d'error informem que el pacient està ingresat
            }

            return isadmitted;

        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}