﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class MedicamentController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public MedicamentController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllMedicament());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Medicament Medicament)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewMedicament(Medicament);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Medicament);
        }

        public ActionResult EditTest(int id)
        {
            var Medicament = _repository.GetMedicamentByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Medicament);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Medicament Medicament)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteMedicament(Medicament.ID_Medicament);
                    _repository.CreateNewMedicament(Medicament);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Medicament);
        }

        public ActionResult DeleteTest(int id)
        {
            var Medicament = _repository.GetMedicamentByID(id);
            ViewBag.Message = "Eliminació";
            return View(Medicament);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Medicament Medicament)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteMedicament(Medicament.ID_Medicament);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Medicament);
        }


        //fi test
        //
        // GET: /Medicament/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();


                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var medicament = from s in db.Medicament
                                     select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        medicament = medicament.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Medicament":
                            medicament = medicament.OrderBy(s => s.ID_Medicament);
                            break;
                        case "Descripcio":
                            medicament = medicament.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            medicament = medicament.OrderBy(s => s.ID_Medicament);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(medicament.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Medicament/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Medicament medicament = db.Medicament.Find(id);
                    if (medicament == null)
                    {
                        return HttpNotFound();
                    }
                    return View(medicament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Medicament/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Medicament/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Medicament medicament)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Medicament.Add(medicament);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(medicament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Medicament/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Medicament medicament = db.Medicament.Find(id);
                    if (medicament == null)
                    {
                        return HttpNotFound();
                    }
                    return View(medicament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Medicament/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Medicament medicament)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Entry(medicament).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    return View(medicament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Medicament/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Medicament medicament = db.Medicament.Find(id);
                    if (medicament == null)
                    {
                        return HttpNotFound();
                    }
                    return View(medicament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Medicament/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "3"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Medicament medicament = db.Medicament.Find(id);
                    db.Medicament.Remove(medicament);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(medicament);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}