﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;
using System.Data.Objects.SqlClient;

namespace GestioHospitalaria.Controllers
{
    public class OrdreController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public OrdreController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllOrdre());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Ordre Ordre)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewOrdre(Ordre);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Ordre);
        }

        public ActionResult EditTest(int id)
        {
            var Ordre = _repository.GetOrdreByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Ordre);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Ordre Ordre)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteOrdre(Ordre.ID_Ordre);
                    _repository.CreateNewOrdre(Ordre);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Ordre);
        }

        public ActionResult DeleteTest(int id)
        {
            var Ordre = _repository.GetOrdreByID(id);
            ViewBag.Message = "Eliminació";
            return View(Ordre);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Ordre Ordre)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteOrdre(Ordre.ID_Ordre);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Ordre);
        }


        //fi test
        //
        // GET: /Ordre/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var ordre = db.Ordre.Include(o => o.TipusOrdre).Include(o => o.Pacient).Include(o => o.Usuari);
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        ordre = ordre.Where(s => s.Pacient.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Pacient.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Pacient.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Ordre":
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                        case "DataHoraInici":
                            ordre = ordre.OrderBy(s => s.DataHoraInici);
                            break;
                        case "Nom_Pacient":
                            ordre = ordre.OrderBy(s => s.Pacient.Nom);
                            break;
                        case "ID_TipusOrdre":
                            ordre = ordre.OrderBy(s => s.ID_TipusOrdre);
                            break;
                        default:
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(ordre.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //vista especial per a veure les ordres d'hospitalització pendents d'un pacient
        public ActionResult OrdresHospitalitzacio(string sortOrder, int id, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var ordre = db.Ordre.Where(o => !db.Ingres.Any(i => o.ID_Ordre == i.ID_Ordre)).Where(o => o.ID_TipusOrdre == 1 && o.ID_Pacient == id);
                    switch (sortOrder)
                    {
                        case "ID_Ordre":
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                        case "DataHoraInici":
                            ordre = ordre.OrderBy(s => s.DataHoraInici);
                            break;
                        case "Nom_Pacient":
                            ordre = ordre.OrderBy(s => s.Pacient.Nom);
                            break;
                        case "ID_TipusOrdre":
                            ordre = ordre.OrderBy(s => s.ID_TipusOrdre);
                            break;
                        default:
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);
                    ViewBag.Direccio = "../Ingres/Create";
                    return View(ordre.ToPagedList(pageNumber, pageSize));
                }
                else
                {

                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //vista especial per a veure les ordres d'operació pendents d'un pacient
        public ActionResult OrdresOperacio(string sortOrder, int id, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var ordre = db.Ordre.Where(o => !db.Reserva.Any(r => o.ID_Ordre == r.ID_Ordre)).Where(o => o.ID_TipusOrdre == 2 && o.ID_Pacient == id);

                    switch (sortOrder)
                    {
                        case "ID_Ordre":
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                        case "DataHoraInici":
                            ordre = ordre.OrderBy(s => s.DataHoraInici);
                            break;
                        case "Nom_Pacient":
                            ordre = ordre.OrderBy(s => s.Pacient.Nom);
                            break;
                        case "ID_TipusOrdre":
                            ordre = ordre.OrderBy(s => s.ID_TipusOrdre);
                            break;
                        default:
                            ordre = ordre.OrderBy(s => s.ID_Ordre);
                            break;
                    }
                    int pageSize = 10;
                    int pageNumber = (page ?? 1); ;
                    ViewBag.Direccio = "../Reserva/Create";
                    return View(ordre.ToPagedList(pageNumber, pageSize));
                }
                else
                {

                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ordre/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Ordre ordre = db.Ordre.Find(id);
                    if (ordre == null)
                    {
                        return HttpNotFound();
                    }
                    return View(ordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ordre/Create

        //obliquem a escollir a un pacient a l'hora de crear una nova ordre
        public ActionResult Create(string id = "")
        {
            try
            {
                    if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                    {
                        if (id != "")//mirem si hem seleccionat pacient
                        {
                            db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                            var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari
                            ViewBag.ID_TipusOrdre = new SelectList(db.TipusOrdre, "ID_TipusOrdre", "Descripcio");

                            var patientName =
                              db.Pacient
                                .Select(s => new
                                {
                                    ID_Pacient = s.ID_Pacient,
                                    Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                                })
                                .ToList();
                            ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", id);
                            //només els metges poden crear un tractament
                            ViewBag.IDPacient = id;
                            ViewBag.IDUsuari = u.ID_Usuari;
                            ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", u.ID_Usuari);
                            return View();
                        }
                        else
                        {
                            Session["Direccio"] = "../Ordre/Create";
                            return RedirectToAction("Index", "Pacient");

                        }
                    }
                    else
                    {
                        return RedirectToAction("Logout", "Account");
                    }

            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ordre/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Ordre ordre)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari
                    if (ModelState.IsValid)
                    {
                        db.Ordre.Add(ordre);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ViewBag.ID_TipusOrdre = new SelectList(db.TipusOrdre, "ID_TipusOrdre", "Descripcio", ordre.ID_TipusOrdre);

                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", ordre.ID_Pacient);

                    //només els metges poden crear un tractament

                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", ordre.ID_Usuari);
                    ViewBag.IDPacient = ordre.ID_Pacient;
                    ViewBag.IDUsuari = ordre.ID_Usuari;
                    return View(ordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ordre/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Ordre ordre = db.Ordre.Find(id);
                    if (ordre == null)
                    {
                        return HttpNotFound();
                    }
                    ViewBag.ID_TipusOrdre = new SelectList(db.TipusOrdre, "ID_TipusOrdre", "Descripcio", ordre.ID_TipusOrdre);

                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", ordre.ID_Pacient);

                    //només els metges poden crear un tractament

                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", ordre.ID_Usuari);
                    ViewBag.IDPacient = ordre.ID_Pacient;
                    ViewBag.IDUsuari = ordre.ID_Usuari;

                    return View(ordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ordre/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Ordre ordre)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid && !isInUse(ordre.ID_Ordre))
                    {
                        db.Entry(ordre).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    
                    ViewBag.ID_TipusOrdre = new SelectList(db.TipusOrdre, "ID_TipusOrdre", "Descripcio", ordre.ID_TipusOrdre);

                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", ordre.ID_Pacient);

                    //només els metges poden crear un tractament

                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", ordre.ID_Usuari);
                    ViewBag.IDPacient = ordre.ID_Pacient;
                    ViewBag.IDUsuari = ordre.ID_Usuari;
                    ModelState.AddModelError("", "No es pot modificar el registre, està en us");
                    return View(ordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Ordre/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Ordre ordre = db.Ordre.Find(id);
                    if (ordre == null)
                    {
                        return HttpNotFound();
                    }
                    return View(ordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Ordre/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Ordre ordre = db.Ordre.Find(id);
                    if (!isInUse(ordre.ID_Ordre))
                    {

                        db.Ordre.Remove(ordre);
                        try
                        {
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        catch
                        {
                            ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                            return View(ordre);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", "No es pot eliminar el registre, està en us");
                        return View(ordre);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //ens servirà per verificar que el registre no està en us, d'aquesta manera no es podrà ni eliminar ni editar
        public Boolean isInUse(int id_registre)
        {
            Boolean sortida = false;
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                var ingres = db.Ingres.Where(i => i.ID_Ordre == id_registre);
                var reserva = db.Reserva.Where(r => r.ID_Ordre == id_registre);

                if (ingres.Count() > 0 || reserva.Count()>0) sortida = true;//si existex un d'aquest registres, signifia que està en us


                return sortida;
            }
            catch
            {
                return true;//en cas de fallida indiquem que el registre està en us...
            }

        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}