﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class PacientController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public PacientController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllPacient());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Pacient Pacient)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewPacient(Pacient);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Pacient);
        }

        public ActionResult EditTest(int id)
        {
            var Pacient = _repository.GetPacientByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Pacient);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Pacient Pacient)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeletePacient(Pacient.ID_Pacient);
                    _repository.CreateNewPacient(Pacient);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Pacient);
        }

        public ActionResult DeleteTest(int id)
        {
            var Pacient = _repository.GetPacientByID(id);
            ViewBag.Message = "Eliminació";
            return View(Pacient);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Pacient Pacient)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeletePacient(Pacient.ID_Pacient);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Pacient);
        }


        //fi test

        //
        // GET: /Pacient/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                //tenen accés a aquesta plana també el metge
                //es farà servir aquesta pàgina per a cercar el pacient i sels donarà accés
                //per crear un tractament.
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4") || Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;
                    ViewBag.Direccio = Auxiliar.Auxiliar.setDireccio(Session);
                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var pacient = from s in db.Pacient
                                  select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        pacient = pacient.Where(s => s.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Pacient":
                            pacient = pacient.OrderBy(s => s.ID_Pacient);
                            break;
                        case "Nom":
                            pacient = pacient.OrderBy(s => s.Nom);
                            break;
                        case "Cognom1":
                            pacient = pacient.OrderBy(s => s.Cognom1);
                            break;
                        case "Cognom2":
                            pacient = pacient.OrderBy(s => s.Cognom2);
                            break;
                        default:
                            pacient = pacient.OrderBy(s => s.ID_Pacient);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(pacient.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Pacient/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Pacient pacient = db.Pacient.Find(id);
                    if (pacient == null)
                    {
                        return HttpNotFound();
                    }
                    return View(pacient);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Pacient/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Pacient/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Pacient pacient)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        string cadenaError = string.Empty;
                        string cadena = String.IsNullOrEmpty(pacient.Nif) ? "0" : pacient.Nif;

                        //verifiquem el nif
                        if (!Auxiliar.Auxiliar.VerificarNIF(cadena)) cadenaError = "El NIF introduït no és correcte";
                        //podem verificar altres errors.....
                        //en cas de produïr-se el missatge es va agregant a cadenaError....
                        //
                        if (string.IsNullOrEmpty(cadenaError) || String.IsNullOrEmpty(pacient.Nif))
                        {
                            db.Pacient.Add(pacient);
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        else
                        {
                            ModelState.AddModelError("", cadenaError);
                            return View(pacient);
                        }


                    }

                    return View(pacient);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Pacient/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Pacient pacient = db.Pacient.Find(id);
                    if (pacient == null)
                    {
                        return HttpNotFound();
                    }
                    return View(pacient);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Pacient/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Pacient pacient)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        string cadenaError = string.Empty;
                        string cadena = String.IsNullOrEmpty(pacient.Nif) ? "0" : pacient.Nif;

                        //verifiquem el nif
                        if (!Auxiliar.Auxiliar.VerificarNIF(cadena)) cadenaError = "El NIF introduït no és correcte";
                        //podem verificar altres errors.....
                        //en cas de produïr-se el missatge es va agregant a cadenaError....
                        //
                        if (string.IsNullOrEmpty(cadenaError) || String.IsNullOrEmpty(pacient.Nif))
                        {
                            db.Entry(pacient).State = EntityState.Modified;
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        else
                        {
                            ModelState.AddModelError("", cadenaError);
                            return View(pacient);
                        }

                    }
                    return View(pacient);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Pacient/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Pacient pacient = db.Pacient.Find(id);
                    if (pacient == null)
                    {
                        return HttpNotFound();
                    }
                    return View(pacient);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Pacient/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Pacient pacient = db.Pacient.Find(id);
                    db.Pacient.Remove(pacient);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(pacient);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}