﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class QuirofanController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public QuirofanController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllQuirofan());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Quirofan Quirofan)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewQuirofan(Quirofan);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Quirofan);
        }

        public ActionResult EditTest(int id)
        {
            var Quirofan = _repository.GetQuirofanByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Quirofan);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Quirofan Quirofan)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteQuirofan(Quirofan.ID_Quirofan);
                    _repository.CreateNewQuirofan(Quirofan);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Quirofan);
        }

        public ActionResult DeleteTest(int id)
        {
            var Quirofan = _repository.GetQuirofanByID(id);
            ViewBag.Message = "Eliminació";
            return View(Quirofan);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Quirofan Quirofan)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteQuirofan(Quirofan.ID_Quirofan);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Quirofan);
        }


        //fi test
        //
        // GET: /Quirofan/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var quirofan = from s in db.Quirofan
                                   select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        quirofan = quirofan.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Quirofan":
                            quirofan = quirofan.OrderBy(s => s.ID_Quirofan);
                            break;
                        case "Descripcio":
                            quirofan = quirofan.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            quirofan = quirofan.OrderBy(s => s.ID_Quirofan);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(quirofan.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Quirofan/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Quirofan quirofan = db.Quirofan.Find(id);
                    if (quirofan == null)
                    {
                        return HttpNotFound();
                    }
                    return View(quirofan);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Quirofan/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Quirofan/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Quirofan quirofan)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid)
                    {
                        db.Quirofan.Add(quirofan);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(quirofan);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Quirofan/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Quirofan quirofan = db.Quirofan.Find(id);
                    if (quirofan == null)
                    {
                        return HttpNotFound();
                    }
                    return View(quirofan);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Quirofan/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Quirofan quirofan)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid)
                    {
                        db.Entry(quirofan).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    return View(quirofan);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Quirofan/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Quirofan quirofan = db.Quirofan.Find(id);
                    if (quirofan == null)
                    {
                        return HttpNotFound();
                    }
                    return View(quirofan);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Quirofan/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Quirofan quirofan = db.Quirofan.Find(id);
                    db.Quirofan.Remove(quirofan);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(quirofan);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try{
            db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

            db.Dispose();
            base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}