﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class ReservaController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public ReservaController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllReserva());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Reserva Reserva)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewReserva(Reserva);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Reserva);
        }

        public ActionResult EditTest(int id)
        {
            var Reserva = _repository.GetReservaByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Reserva);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Reserva Reserva)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteReserva(Reserva.ID_Reserva);
                    _repository.CreateNewReserva(Reserva);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Reserva);
        }

        public ActionResult DeleteTest(int id)
        {
            var Reserva = _repository.GetReservaByID(id);
            ViewBag.Message = "Eliminació";
            return View(Reserva);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Reserva Reserva)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteReserva(Reserva.ID_Reserva);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Reserva);
        }


        //fi test

        //
        // GET: /Reserva/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var reserva = db.Reserva.Include(r => r.EstatReserva).Include(r => r.Ordre).Include(r => r.Quirofan).Include(r => r.Usuari);
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        reserva = reserva.Where(s => s.Ordre.Usuari.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Usuari.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Ordre.Usuari.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Reserva":
                            reserva = reserva.OrderBy(s => s.ID_Reserva);
                            break;
                        case "ID_Quirofan":
                            reserva = reserva.OrderBy(s => s.ID_Quirofan);
                            break;
                        case "DataHoraInici":
                            reserva = reserva.OrderBy(s => s.DataHoraInici);
                            break;
                        case "DataHoraFi":
                            reserva = reserva.OrderBy(s => s.DataHoraFi);
                            break;
                        default:
                            reserva = reserva.OrderBy(s => s.ID_Reserva);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(reserva.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }


        //
        // GET: /Reserva/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Reserva reserva = db.Reserva.Find(id);
                    if (reserva == null)
                    {
                        return HttpNotFound();
                    }
                    return View(reserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Reserva/Create

        public ActionResult Create(string id = "")
        {
            try
            {
                    if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                    {
                        if (id != "")//mirem si hem seleccionat una ordre d'hospitalització
                        {
                            db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                            var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari
                            Ordre ordre = db.Ordre.Find(Convert.ToInt32(id));//busquem les dades de l'ordre

                            //cerquem un quiròfan disponible
                            DateTime dInici = ordre.DataHoraInici;
                            DateTime dFi = ordre.DataHoraInici.AddHours(ordre.Durada);
                            var quirofan = db.Quirofan.Where(q => !db.Reserva.Any(r => q.ID_Quirofan == r.ID_Quirofan
                                && ((r.DataHoraInici < dFi && r.DataHoraFi > dFi)//la data fi es troba entre les dates d'una reserva
                                || (r.DataHoraInici < dInici && r.DataHoraFi > dInici)//la data inici es troba dintre de les dates d'una reserva
                                || (r.DataHoraInici > dInici && r.DataHoraFi < dFi))));//les dates inici i fi  de l'ordre son superiors a les reserves existents

                            if (quirofan.Count() > 0)//tenim quiròfan lliure
                            {
                                ViewBag.IDUsuari = u.ID_Usuari;
                                ViewBag.IDOrdre = id;
                                ViewBag.ID_EstatReserva = new SelectList(db.EstatReserva, "ID_EstatReserva", "Descripcio", 1);
                                ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", id);
                                ViewBag.IDQuirofan = quirofan.First().ID_Quirofan;//donem el primer quiròfan lliure
                                ViewBag.ID_Quirofan = new SelectList(db.Quirofan, "ID_Quirofan", "Descripcio", quirofan.First().ID_Quirofan);
                                ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", u.ID_Usuari);
                                ViewBag.DataInici = ordre.DataHoraInici.ToString();
                                ViewBag.DataFi = ordre.DataHoraInici.AddHours(ordre.Durada).ToString();
                                ViewBag.IDEstatReserva = 1;//forçem l'estat reserva
                                return View();
                            }
                            else
                            {

                                Session["Error"] = "No queden quirofans disponibles per aquestes dates";

                                return RedirectToAction("Index", "Pacient");
                            }
                        }
                        else//començem el procés de selecció d'ordre
                        {
                            return RedirectToAction("Index", "Pacient");

                        }
                    }
                    else
                    {
                        return RedirectToAction("Logout", "Account");
                    }

            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Reserva/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Reserva reserva)
        {//COM QUE NO HI HA CAP VARIACIÓ DE DADES POSSIBLES
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Reserva.Add(reserva);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    ViewBag.IDUsuari = reserva.ID_Usuari;
                    ViewBag.IDOrdre = reserva.ID_Ordre;
                    ViewBag.DataInici = reserva.DataHoraInici.ToString();
                    ViewBag.DataFi = reserva.DataHoraFi.ToString();
                    ViewBag.IDEstatReserva = 1;//forçem l'estat reserva
                    ViewBag.IDQuirofan = reserva.ID_Quirofan;

                    ViewBag.ID_EstatReserva = new SelectList(db.EstatReserva, "ID_EstatReserva", "Descripcio", reserva.ID_EstatReserva);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", reserva.ID_Ordre);
                    ViewBag.ID_Quirofan = new SelectList(db.Quirofan, "ID_Quirofan", "Descripcio", reserva.ID_Quirofan);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", reserva.ID_Usuari);
                    return View(reserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Reserva/Edit/5

        public ActionResult Edit(int id = 0)
        {//Podem canviar el quiròfan
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Reserva reserva = db.Reserva.Find(id);
                    if (reserva == null)
                    {
                        return HttpNotFound();
                    }
                    ViewBag.IDUsuari = reserva.ID_Usuari;
                    ViewBag.IDOrdre = reserva.ID_Ordre;
                    ViewBag.DataInici = reserva.DataHoraInici.ToString();
                    ViewBag.DataFi = reserva.DataHoraFi.ToString();
                    ViewBag.IDEstatReserva = 1;//forçem l'estat reserva
                    ViewBag.IDQuirofan = reserva.ID_Quirofan;
                    //cerquem quiròfan lliure incloent el reservat per aquesta
                    DateTime dInici = reserva.DataHoraInici;
                    DateTime dFi = reserva.DataHoraFi;
                    var quirofan = db.Quirofan.Where(q => !db.Reserva.Any(r => q.ID_Quirofan == r.ID_Quirofan && r.ID_Reserva != reserva.ID_Reserva
                        && ((r.DataHoraInici < dFi && r.DataHoraFi > dFi)//la data fi es troba entre les dates d'una reserva
                        || (r.DataHoraInici < dInici && r.DataHoraFi > dInici)//la data inici es troba dintre de les dates d'una reserva
                        || (r.DataHoraInici > dInici && r.DataHoraFi < dFi))));//les dates inici i fi  de l'ordre son superiors a les reserves existents


                    ViewBag.ID_EstatReserva = new SelectList(db.EstatReserva, "ID_EstatReserva", "Descripcio", reserva.ID_EstatReserva);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", reserva.ID_Ordre);
                    ViewBag.ID_Quirofan = new SelectList(quirofan, "ID_Quirofan", "Descripcio", reserva.ID_Quirofan);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", reserva.ID_Usuari);
                    return View(reserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Reserva/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Reserva reserva)
        {//Podem canviar el quiròfan
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.Entry(reserva).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ViewBag.IDUsuari = reserva.ID_Usuari;
                    ViewBag.IDOrdre = reserva.ID_Ordre;
                    ViewBag.DataInici = reserva.DataHoraInici.ToString();
                    ViewBag.DataFi = reserva.DataHoraFi.ToString();
                    ViewBag.IDEstatReserva = 1;//forçem l'estat reserva
                    ViewBag.IDQuirofan = reserva.ID_Quirofan;
                    //cerquem quiròfan lliure incloent el reservat per aquesta
                    DateTime dInici = reserva.DataHoraInici;
                    DateTime dFi = reserva.DataHoraFi;
                    var quirofan = db.Quirofan.Where(q => !db.Reserva.Any(r => q.ID_Quirofan == r.ID_Quirofan && r.ID_Reserva != reserva.ID_Reserva
                        && ((r.DataHoraInici < dFi && r.DataHoraFi > dFi)//la data fi es troba entre les dates d'una reserva
                        || (r.DataHoraInici < dInici && r.DataHoraFi > dInici)//la data inici es troba dintre de les dates d'una reserva
                        || (r.DataHoraInici > dInici && r.DataHoraFi < dFi))));//les dates inici i fi  de l'ordre son superiors a les reserves existents


                    ViewBag.ID_EstatReserva = new SelectList(db.EstatReserva, "ID_EstatReserva", "Descripcio", reserva.ID_EstatReserva);
                    ViewBag.ID_Ordre = new SelectList(db.Ordre, "ID_Ordre", "ID_Ordre", reserva.ID_Ordre);
                    ViewBag.ID_Quirofan = new SelectList(quirofan, "ID_Quirofan", "Descripcio", reserva.ID_Quirofan);
                    ViewBag.ID_Usuari = new SelectList(db.Usuari, "ID_Usuari", "NomUsuari", reserva.ID_Usuari);
                    return View(reserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Reserva/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Reserva reserva = db.Reserva.Find(id);
                    if (reserva == null)
                    {
                        return HttpNotFound();
                    }
                    return View(reserva);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Reserva/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "4"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Reserva reserva = db.Reserva.Find(id);
                    db.Reserva.Remove(reserva);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(reserva);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}