﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class TipusEmpleatController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public TipusEmpleatController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }

        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllTipusEmpleat());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(TipusEmpleat TipusEmpleat)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewTipusEmpleat(TipusEmpleat);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(TipusEmpleat);
        }

        public ActionResult EditTest(int id)
        {
            var TipusEmpleat = _repository.GetTipusEmpleatByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(TipusEmpleat);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(TipusEmpleat TipusEmpleat)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTipusEmpleat(TipusEmpleat.ID_TipusEmpleat);
                    _repository.CreateNewTipusEmpleat(TipusEmpleat);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(TipusEmpleat);
        }

        public ActionResult DeleteTest(int id)
        {
            var TipusEmpleat = _repository.GetTipusEmpleatByID(id);
            ViewBag.Message = "Eliminació";
            return View(TipusEmpleat);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(TipusEmpleat TipusEmpleat)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTipusEmpleat(TipusEmpleat.ID_TipusEmpleat);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(TipusEmpleat);
        }


        //fi test
        //
        // GET: /TipusEmpleat/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var tipusEmpleat = from s in db.TipusEmpleat
                                       select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        tipusEmpleat = tipusEmpleat.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_TipusEmpleat":
                            tipusEmpleat = tipusEmpleat.OrderBy(s => s.ID_TipusEmpleat);
                            break;
                        case "Descripcio":
                            tipusEmpleat = tipusEmpleat.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            tipusEmpleat = tipusEmpleat.OrderBy(s => s.ID_TipusEmpleat);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(tipusEmpleat.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusEmpleat/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    TipusEmpleat tipusempleat = db.TipusEmpleat.Find(id);
                    if (tipusempleat == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusempleat);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusEmpleat/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusEmpleat/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(TipusEmpleat tipusempleat)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid)
                    {
                        db.TipusEmpleat.Add(tipusempleat);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(tipusempleat);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusEmpleat/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    TipusEmpleat tipusempleat = db.TipusEmpleat.Find(id);
                    if (tipusempleat == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusempleat);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusEmpleat/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(TipusEmpleat tipusempleat)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    if (ModelState.IsValid && tipusempleat.ID_TipusEmpleat > 4)//evitem modificar dades reservades
                    {
                        db.Entry(tipusempleat).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ModelState.AddModelError("", "No es pot modificar una dada reservada del sistema");
                    return View(tipusempleat);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusEmpleat/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    TipusEmpleat tipusempleat = db.TipusEmpleat.Find(id);
                    if (tipusempleat == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusempleat);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusEmpleat/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    TipusEmpleat tipusempleat = db.TipusEmpleat.Find(id);
                    if (tipusempleat.ID_TipusEmpleat > 4)//evitem esborrar dades reservades
                    {

                        db.TipusEmpleat.Remove(tipusempleat);
                        try
                        {
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        catch
                        {
                            ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                            return View(tipusempleat);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", "No es pot esborrar una dada reservada del sistema");
                        return View(tipusempleat);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}