﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class TipusOrdreController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public TipusOrdreController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllTipusOrdre());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(TipusOrdre TipusOrdre)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewTipusOrdre(TipusOrdre);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(TipusOrdre);
        }

        public ActionResult EditTest(int id)
        {
            var TipusOrdre = _repository.GetTipusOrdreByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(TipusOrdre);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(TipusOrdre TipusOrdre)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTipusOrdre(TipusOrdre.ID_TipusOrdre);
                    _repository.CreateNewTipusOrdre(TipusOrdre);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(TipusOrdre);
        }

        public ActionResult DeleteTest(int id)
        {
            var TipusOrdre = _repository.GetTipusOrdreByID(id);
            ViewBag.Message = "Eliminació";
            return View(TipusOrdre);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(TipusOrdre TipusOrdre)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTipusOrdre(TipusOrdre.ID_TipusOrdre);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(TipusOrdre);
        }


        //fi test
        //
        // GET: /TipusOrdre/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {

                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var tipusOrdre = from s in db.TipusOrdre
                                     select s;
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        tipusOrdre = tipusOrdre.Where(s => s.Descripcio.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_TipusOrdre":
                            tipusOrdre = tipusOrdre.OrderBy(s => s.ID_TipusOrdre);
                            break;
                        case "Descripcio":
                            tipusOrdre = tipusOrdre.OrderBy(s => s.Descripcio);
                            break;
                        default:
                            tipusOrdre = tipusOrdre.OrderBy(s => s.ID_TipusOrdre);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(tipusOrdre.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusOrdre/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    TipusOrdre tipusordre = db.TipusOrdre.Find(id);
                    if (tipusordre == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusOrdre/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusOrdre/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(TipusOrdre tipusordre)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        db.TipusOrdre.Add(tipusordre);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    return View(tipusordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusOrdre/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    TipusOrdre tipusordre = db.TipusOrdre.Find(id);
                    if (tipusordre == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusOrdre/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(TipusOrdre tipusordre)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid && tipusordre.ID_TipusOrdre > 2)//evitem modificar dades reservades
                    {
                        db.Entry(tipusordre).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    ModelState.AddModelError("", "No es pot modificar una dada reservada del sistema");
                    return View(tipusordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /TipusOrdre/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    TipusOrdre tipusordre = db.TipusOrdre.Find(id);
                    if (tipusordre == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tipusordre);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /TipusOrdre/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    TipusOrdre tipusordre = db.TipusOrdre.Find(id);
                    if (tipusordre.ID_TipusOrdre > 2)//evitem esborrar dades reservades
                    {
                        db.TipusOrdre.Remove(tipusordre);
                        try
                        {
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        catch
                        {
                            ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                            return View(tipusordre);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", "No es pot esborrar una dada reservada del sistema");
                        return View(tipusordre);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}