﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;
using System.Data.Objects.SqlClient;

namespace GestioHospitalaria.Controllers
{
    public class TractamentController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public TractamentController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllTractament());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Tractament Tractament)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewTractament(Tractament);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Tractament);
        }

        public ActionResult EditTest(int id)
        {
            var Tractament = _repository.GetTractamentByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Tractament);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Tractament Tractament)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTractament(Tractament.ID_Tractament);
                    _repository.CreateNewTractament(Tractament);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Tractament);
        }

        public ActionResult DeleteTest(int id)
        {
            var Tractament = _repository.GetTractamentByID(id);
            ViewBag.Message = "Eliminació";
            return View(Tractament);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Tractament Tractament)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteTractament(Tractament.ID_Tractament);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Tractament);
        }


        //fi test
        //
        // GET: /Tractament/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var tractament = db.Tractament.Include(t => t.Dosi).Include(t => t.Frequencia).Include(t => t.Medicament).Include(t => t.Pacient).Include(t => t.Usuari);
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }

                    if (!String.IsNullOrEmpty(searchString))//busquem per pacient
                    {
                        tractament = tractament.Where(s => s.Pacient.Nom.ToUpper().Contains(searchString.ToUpper()) || s.Pacient.Cognom1.ToUpper().Contains(searchString.ToUpper()) || s.Pacient.Cognom2.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Pacient":
                            tractament = tractament.OrderBy(s => s.Pacient.ID_Pacient);
                            break;
                        case "Nom_Pacient":
                            tractament = tractament.OrderBy(s => s.Pacient.Nom);
                            break;
                        case "ID_Tractament":
                            tractament = tractament.OrderBy(s => s.ID_Tractament);
                            break;
                        case "DataInici":
                            tractament = tractament.OrderBy(s => s.DataInici);
                            break;
                        case "DataFi":
                            tractament = tractament.OrderBy(s => s.DataFi);
                            break;
                        default:
                            tractament = tractament.OrderBy(s => s.ID_Tractament);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(tractament.ToPagedList(pageNumber, pageSize));


                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }


        //
        // GET: /Tractament/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Tractament tractament = db.Tractament.Find(id);
                    if (tractament == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tractament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Tractament/Create
        //obliquem a escollir a un pacient a l'hora de crear un nou tractament
        public ActionResult Create(string  id = "")
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    if (id != "")//mirem si hem seleccionat pacient
                    {

                        db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                        var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari
                        var dosiName =
                        db.Dosi
                         .Select(d => new
                         {
                             ID_Dosi = d.ID_Dosi,
                             Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                         })
                         .ToList();

                        ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat");
                        ViewBag.ID_Frequencia = new SelectList(db.Frequencia, "ID_Frequencia", "Descripcio");
                        ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio");

                        var patientName =
                          db.Pacient
                            .Select(s => new
                            {
                                ID_Pacient = s.ID_Pacient,
                                Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                            })
                            .ToList();
                        ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", id);
                        //només els metges poden crear un tractament i seleccionem directament l'usuari
                        ViewBag.IDPacient = id;
                        ViewBag.IDUsuari = u.ID_Usuari;
                        ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", u.ID_Usuari);
                        return View();
                    }
                    else
                    {
                        Session["Direccio"] = "../Tractament/Create";
                        return RedirectToAction("Index", "Pacient");

                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }

            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Tractament/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Tractament tractament)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    var u = db.Usuari.Where(a => a.NomUsuari == User.Identity.Name).First();//necessitem el ID d'usuari
                    //verifiquem les dates
                    if (DateTime.Compare(tractament.DataFi, tractament.DataInici) < 1)
                    {
                        ModelState.AddModelError("", "La data fi ha de ser major a la data inici");
                    }
                    if (ModelState.IsValid)
                    {
                        db.Tractament.Add(tractament);
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }

                    var dosiName =
                    db.Dosi
                    .Select(d => new
                    {
                        ID_Dosi = d.ID_Dosi,
                        Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                    })
                    .ToList();
                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", tractament.ID_Dosi);
                    ViewBag.ID_Frequencia = new SelectList(db.Frequencia, "ID_Frequencia", "Descripcio", tractament.ID_Frequencia);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", tractament.ID_Medicament);
                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", tractament.ID_Pacient);
                    //només els metges poden crear un tractament
                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", tractament.ID_Usuari);
                    ViewBag.IDPacient = tractament.ID_Pacient;
                    ViewBag.IDUsuari = tractament.ID_Usuari;
                    return View(tractament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Tractament/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Tractament tractament = db.Tractament.Find(id);
                    if (tractament == null)
                    {
                        return HttpNotFound();
                    }
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();
                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", tractament.ID_Dosi);
                    ViewBag.ID_Frequencia = new SelectList(db.Frequencia, "ID_Frequencia", "Descripcio", tractament.ID_Frequencia);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", tractament.ID_Medicament);
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", tractament.ID_Pacient);
                    //només els metges poden crear un tractament
                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", tractament.ID_Usuari);
                    ViewBag.IDPacient = tractament.ID_Pacient;
                    ViewBag.IDUsuari = tractament.ID_Usuari;
                    return View(tractament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Tractament/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Tractament tractament)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    //verifiquem les dates
                    if (DateTime.Compare(tractament.DataFi, tractament.DataInici) < 1)
                    {
                        ModelState.AddModelError("", "La data fi ha de ser major a la data inici");
                    }
                    if (ModelState.IsValid)
                    {
                        db.Entry(tractament).State = EntityState.Modified;
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    var dosiName =
                    db.Dosi
                     .Select(d => new
                     {
                         ID_Dosi = d.ID_Dosi,
                         Unitat = SqlFunctions.StringConvert((Double)d.Valor).Trim() + "-" + d.Unitat
                     })
                     .ToList();
                    var patientName =
                      db.Pacient
                        .Select(s => new
                        {
                            ID_Pacient = s.ID_Pacient,
                            Descripcio = SqlFunctions.StringConvert((Double)s.ID_Pacient).Trim() + "-" + s.Nom
                        })
                        .ToList();
                    ViewBag.ID_Dosi = new SelectList(dosiName, "ID_Dosi", "Unitat", tractament.ID_Dosi);
                    ViewBag.ID_Frequencia = new SelectList(db.Frequencia, "ID_Frequencia", "Descripcio", tractament.ID_Frequencia);
                    ViewBag.ID_Medicament = new SelectList(db.Medicament, "ID_Medicament", "Descripcio", tractament.ID_Medicament);
                    ViewBag.ID_Pacient = new SelectList(patientName, "ID_Pacient", "Descripcio", tractament.ID_Pacient);
                    ViewBag.ID_Usuari = new SelectList(from s in db.Usuari where s.ID_TipusEmpleat == 2 select s, "ID_Usuari", "NomUsuari", tractament.ID_Usuari);
                    ViewBag.IDPacient = tractament.ID_Pacient;
                    ViewBag.IDUsuari = tractament.ID_Usuari;
                    return View(tractament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Tractament/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Tractament tractament = db.Tractament.Find(id);
                    if (tractament == null)
                    {
                        return HttpNotFound();
                    }
                    return View(tractament);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Tractament/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "2"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Tractament tractament = db.Tractament.Find(id);
                    db.Tractament.Remove(tractament);
                    try
                    {
                        db.SaveChanges();
                        return RedirectToAction("Index");
                    }
                    catch
                    {
                        ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                        return View(tractament);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
        }
    }
}