﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using GestioHospitalaria.Models;
using GestioHospitalaria.Auxiliar;
using PagedList;

namespace GestioHospitalaria.Controllers
{
    public class UsuariController : Controller
    {
        private BD_GestioHospitalariaEntities db;
        IGestioHospitalariaRepository _repository;
        //L'arxiu StructureMaps.xml determinarà el comportament de repository
        public UsuariController(IGestioHospitalariaRepository repository)
        {
            _repository = repository;
            
        }
        //inici test
        public ViewResult IndexListTest()
        {
            ViewBag.Message = "Index List Test";
            return View("IndexListTest", _repository.GetAllUsuari());
        }
        public ActionResult CreateTest()
        {
            ViewBag.Message = "Entrada Creació";
            return View("CreateTest");
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult CreateTest(Usuari Usuari)
        {
            try
            {
                if (ModelState.IsValid)//si no tenim errors en la verificació
                {
                    _repository.CreateNewUsuari(Usuari);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la creació";
            }

            ViewBag.Message = "Error en la creació";
            return View(Usuari);
        }

        public ActionResult EditTest(int id)
        {
            var Usuari = _repository.GetUsuariByID(id);
            ViewBag.Message = "Entrada Edició";
            return View(Usuari);
        }

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult EditTest(Usuari Usuari)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteUsuari(Usuari.ID_Usuari);
                    _repository.CreateNewUsuari(Usuari);
                    _repository.SaveChanges();

                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en la Edició";
            }

            ViewBag.Message = "Error en la Edició";
            return View(Usuari);
        }

        public ActionResult DeleteTest(int id)
        {
            var Usuari = _repository.GetUsuariByID(id);
            ViewBag.Message = "Eliminació";
            return View(Usuari);
        }

        //
        // POST: /Home/Delete/5

        [HttpPost]
        public ActionResult DeleteTest(Usuari Usuari)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    _repository.DeleteUsuari(Usuari.ID_Usuari);
                    _repository.SaveChanges();
                    return RedirectToAction("IndexListTest");
                }
            }
            catch (Exception ex)
            {
                ModelState.AddModelError("", ex);
                ViewBag.Message = "Error en l'eliminació";
            }
            ViewBag.Message = "Error en l'eliminació";
            return View(Usuari);
        }


        //fi test
        //
        // GET: /Usuari/

        public ActionResult Index(string sortOrder, string currentFilter, string searchString, int? page)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();


                    ViewBag.CurrentSort = sortOrder;

                    ViewBag.NameSortParm = String.IsNullOrEmpty(sortOrder) ? sortOrder : "";
                    var usuari = db.Usuari.Include(u => u.TipusEmpleat);
                    if (searchString != null)
                    {
                        page = 1;
                    }
                    else
                    {
                        searchString = currentFilter;
                    }


                    if (!String.IsNullOrEmpty(searchString))
                    {
                        usuari = usuari.Where(s => s.NomUsuari.ToUpper().Contains(searchString.ToUpper()));
                    }


                    switch (sortOrder)
                    {
                        case "ID_Usuari":
                            usuari = usuari.OrderBy(s => s.ID_Usuari);
                            break;
                        case "ID_TipusEmpleat":
                            usuari = usuari.OrderBy(s => s.ID_TipusEmpleat);
                            break;
                        case "NomUsuari":
                            usuari = usuari.OrderBy(s => s.NomUsuari);
                            break;
                        default:
                            usuari = usuari.OrderBy(s => s.ID_Usuari);
                            break;
                    }

                    int pageSize = 10;
                    int pageNumber = (page ?? 1);

                    return View(usuari.ToPagedList(pageNumber, pageSize));
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Usuari/Details/5

        public ActionResult Details(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                    Usuari usuari = db.Usuari.Find(id);
                    if (usuari == null)
                    {
                        return HttpNotFound();
                    }
                    return View(usuari);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Usuari/Create

        public ActionResult Create()
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio");
                    return View();
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Usuari/Create

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Create(Usuari usuari)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {
                        string cadenaError = string.Empty;
                        string cadena = String.IsNullOrEmpty(usuari.Nif) ? "0" : usuari.Nif;

                        //verifiquem el nif
                        if (!Auxiliar.Auxiliar.VerificarNIF(cadena)) cadenaError = "El NIF introduït no és correcte";
                        //podem verificar altres errors.....
                        //en cas de produïr-se el missatge es va agregant a cadenaError....
                        //
                        if (string.IsNullOrEmpty(cadenaError) || String.IsNullOrEmpty(usuari.Nif))
                        {
                            usuari.Contrasenya = Auxiliar.Auxiliar.EncodeMD5(usuari.Contrasenya);
                            db.Usuari.Add(usuari);
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        else
                        {
                            ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio", usuari.ID_TipusEmpleat);
                            ModelState.AddModelError("", cadenaError);
                            return View(usuari);
                        }
                    }

                    ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio", usuari.ID_TipusEmpleat);
                    return View(usuari);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Usuari/Edit/5

        public ActionResult Edit(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Usuari usuari = db.Usuari.Find(id);
                    if (usuari == null)
                    {
                        return HttpNotFound();
                    }
                    ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio", usuari.ID_TipusEmpleat);
                    return View(usuari);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Usuari/Edit/5

        [HttpPost]
        [ValidateAntiForgeryToken]
        public ActionResult Edit(Usuari usuari)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    if (ModelState.IsValid)
                    {

                        string cadenaError = string.Empty;
                        string cadena = String.IsNullOrEmpty(usuari.Nif) ? "0" : usuari.Nif;

                        //verifiquem el nif
                        if (!Auxiliar.Auxiliar.VerificarNIF(cadena)) cadenaError = "El NIF introduït no és correcte";
                        //podem verificar altres errors.....
                        //en cas de produïr-se el missatge es va agregant a cadenaError....
                        //
                        if (string.IsNullOrEmpty(cadenaError) || String.IsNullOrEmpty(usuari.Nif))
                        {
                            try
                            {
                                if (usuari.ID_Usuari == 1)
                                {
                                    usuari.ID_TipusEmpleat = 1;//l'usuari administrador sempre serà  de tipus empleat 1
                                    usuari.NomUsuari = "Administrador";//sempre aquest serà el seu nom d'usuari
                                    //la resta de dades les pot canviar totes....
                                    //a excepció de la contrasenya que te el seu propi procediment.
                                }
                                db.Entry(usuari).State = EntityState.Modified;

                                db.SaveChanges();
                                return RedirectToAction("Index");
                            }
                            catch (Exception ex)
                            {
                                ModelState.AddModelError("", ex.Message);
                            }
                        }
                        else
                        {
                            ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio", usuari.ID_TipusEmpleat);
                            ModelState.AddModelError("", cadenaError);
                            return View(usuari);
                        }

                    }
                    ViewBag.ID_TipusEmpleat = new SelectList(db.TipusEmpleat, "ID_TipusEmpleat", "Descripcio", usuari.ID_TipusEmpleat);
                    return View(usuari);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // GET: /Usuari/Delete/5

        public ActionResult Delete(int id = 0)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Usuari usuari = db.Usuari.Find(id);
                    if (usuari == null)
                    {
                        return HttpNotFound();
                    }
                    return View(usuari);
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        //
        // POST: /Usuari/Delete/5

        [HttpPost, ActionName("Delete")]
        [ValidateAntiForgeryToken]
        public ActionResult DeleteConfirmed(int id)
        {
            try
            {
                if (Auxiliar.Auxiliar.is_Authorized(Session, "1"))
                {
                    db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();
                    Usuari usuari = db.Usuari.Find(id);
                    if (usuari.ID_Usuari != 1)//no podem eliminar l'usuari administrador
                    {
                        db.Usuari.Remove(usuari);
                        try
                        {
                            db.SaveChanges();
                            return RedirectToAction("Index");
                        }
                        catch
                        {
                            ModelState.AddModelError("", "No es pot esborrar, la dada està en ús");
                            return View(usuari);
                        }
                    }
                    else
                    {
                        ModelState.AddModelError("", "No es pot esborrar una dada reservada del sistema");
                        return View(usuari);
                    }
                }
                else
                {
                    return RedirectToAction("Logout", "Account");
                }
            }
            catch (Exception ex)
            {
                ViewBag.Message = "Error no controlat";
                Session["ErrorMes"] = ex.Message;
                return RedirectToAction("Logout", "Account");
            }
        }

        protected override void Dispose(bool disposing)
        {
            try
            {
                db = ((EntityGestioHospitalariaManagerRepository)_repository).getDB();

                db.Dispose();
                base.Dispose(disposing);
            }
            catch
            {
            }
         }
    }
}